# coding: utf-8
from abc import ABCMeta
from abc import abstractmethod
from json.decoder import JSONDecodeError
import json

from .exceptions import RawDataLoadException
from .exceptions import ServiceException


def message_to_dict(data: str) -> dict:
    """Преобразует полученное из очереди сообщение к словарю."""
    try:
        return json.loads(data)
    except (JSONDecodeError, TypeError) as exc:
        raise RawDataLoadException(
            f'Ошибка при декодировании данных:\n{str(exc)}.'
        )


class BaseSyncService(metaclass=ABCMeta):
    """Базовый класс сервиса для синхронизации данных между системами."""

    def decode_message(self, message: str) -> dict:
        """Декодирует полученное сообщение."""
        try:
            decoded_data = message_to_dict(message)
        except RawDataLoadException:  # noqa: W0706
            raise
        return decoded_data

    @abstractmethod
    def process_data(self, data: dict) -> None:
        """Обрабатывает полученные данные для синхронизации."""
        raise NotImplementedError

    def sync(self, data: str) -> None:
        """Выполняет синхронизацию данных."""
        decoded_data = self.decode_message(data)
        try:
            self.process_data(decoded_data)
        except ServiceException:  # noqa: W0706
            raise
